# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/routes/bootstrap.ipynb.

# %% auto 0
__all__ = ['GetBootstrap_InvalidAuthMethod', 'get_bootstrap', 'get_bootstrap_features', 'get_bootstrap_pages']

# %% ../../nbs/routes/bootstrap.ipynb 3
import httpx

import domolibrary.client.get_data as gd
import domolibrary.client.ResponseGetData as rgd
import domolibrary.client.DomoAuth as dmda

# %% ../../nbs/routes/bootstrap.ipynb 4
class GetBootstrap_InvalidAuthMethod(Exception):
    def __init__(self, domo_instance):
        message = f"invalid auth method sent to {domo_instance} bootstrap API, use DomoFullAuth (username and password) authentication"
        super().__init__(message)

async def get_bootstrap(
    auth: dmda.DomoFullAuth, ## only works with DomoFullAuth authentication, do not use TokenAuth
    debug_api: bool = False, session: httpx.AsyncClient = None
) -> rgd.ResponseGetData:
    """get bootstrap data"""

    # url = f"https://{auth.domo_instance}.domo.com/api/domoweb/bootstrap?v2Navigation=false"
    url = f"https://{auth.domo_instance}.domo.com/api/domoweb/bootstrap?v2Navigation=true"

    res = await gd.get_data(
        url=url, method="GET", auth=auth, debug_api=debug_api, session=session
    )

    if res.status == 302:
        raise GetBootstrap_InvalidAuthMethod(auth.domo_instance)

    return res


# %% ../../nbs/routes/bootstrap.ipynb 8
async def get_bootstrap_features(   
    auth: dmda.DomoAuth, session: httpx.AsyncClient = None, debug_api: bool = False
) -> rgd.ResponseGetData:
    res = await get_bootstrap(auth=auth, session=session, debug_api=debug_api)

    if not res.is_success:
        return None

    res.response = res.response.get("data").get("features")
    return res

# %% ../../nbs/routes/bootstrap.ipynb 11
async def get_bootstrap_pages(
    auth: dmda.DomoAuth, session: httpx.AsyncClient = None, debug_api: bool = False
) -> rgd.ResponseGetData:
    res = await get_bootstrap(auth=auth, session=session, debug_api=debug_api)

    if not res.is_success:
        return None

    res.response = res.response.get("data").get("pages")
    return res




from pprint import pprint

import aiohttp

from ...utils.ResponseGetData import ResponseGetData
from ..DomoAuth import DomoFullAuth
from .get_data import get_data, looper


async def get_applications(full_auth: DomoFullAuth,
                           session: aiohttp.ClientSession = None,
                           debug: bool = False,
                           log_results: bool = False
                           ):

    url = f'https://{full_auth.domo_instance}.domo.com/api/executor/v1/applications/'

    if debug:
        print(url)

    return await get_data(
        auth=full_auth,
        url=url,
        method='GET',
        log_results=log_results,
        debug=debug,
        session=session
    )


async def get_application_by_id(full_auth: DomoFullAuth,
                                application_id: str,
                                session: aiohttp.ClientSession = None,
                                debug: bool = False, log_results: bool = False):

    url = f'https://{full_auth.domo_instance}.domo.com/api/executor/v1/applications/{application_id}'

    if debug:
        print(url)

    return await get_data(
        auth=full_auth,
        url=url,
        method='GET',
        log_results=log_results,
        debug=debug,
        session=session
    )



import asyncio
import importlib
from dataclasses import dataclass, field
from enum import Enum

import aiohttp

import Library.DomoClasses.DomoAuth as dmda

from .DomoDataset import DomoDataset
from .routes import account_routes, datacenter_routes

importlib.reload(datacenter_routes)


class DomoEntity(Enum):
    DATASET = 'DATA_SOURCE'
    DATAFLOW = 'DATAFLOW'
    PAGE = 'PAGE'
    CARD = 'CARD'
    ACCOUNT = 'ACCOUNT'


class DomoDatacenter:
    full_auth: dmda.DomoFullAuth = None

    @classmethod
    def generate_search_datacenter_body(cls, entities_list: list[str] = ['DATASET'],
                                        filters: list[dict] = None,
                                        combineResults: bool = True,
                                        count: int = 100,
                                        offset: int = 0):
        return {
            "entities": entities_list,
            #old value on Jan 13
            #"entityList": entities_list,
            "filters": filters or [],
            "combineResults": combineResults,
            "query": "*",
            "count": count,
            "offset": offset}

    @classmethod
    def generate_search_datacenter_body_by_name(cls,
                                                entity_name, entities_list: list[str] = ['DATASET'],
                                                filters: list[dict] = None,
                                                combineResults: bool = True,
                                                count: int = 100,
                                                offset: int = 0):

        body = cls.generate_search_datacenter_body(
            entities_list, filters, combineResults, count, offset)
        body['filters'].append(
            {'field': 'name_sort', 'filterType': 'wildcard', 'query': entity_name})
        return body

    @classmethod
    async def search_datacenter(cls, full_auth: dmda.DomoFullAuth,
                                body: dict = None,
                                session: aiohttp.ClientSession = None,
                                maximum: int = None,
                                debug: bool = False, log_result: bool = False) -> list:

        def arr_fn(res):
            # pprint(res.response)
            return res.response.get('searchObjects')

        def alter_maximum_fn(res):
            return res.response.get('totalResultCount')

        if not body:
            body = cls.generate_search_datacenter_body(
                entities_list=['DATASET'],
                filters=[],
                count=1000,
                offset=0,
                combineResults=False
            )

        if debug:
            print(body)

        if maximum:
            return await datacenter_routes.search_datacenter(full_auth=full_auth,
                                                             arr_fn=arr_fn,
                                                             maximum=maximum,
                                                             body=body,
                                                             session=session,
                                                             debug=debug)

        return await datacenter_routes.search_datacenter(full_auth=full_auth,
                                                         arr_fn=arr_fn,
                                                         alter_maximum_fn=alter_maximum_fn,
                                                         body=body,
                                                         session=session,
                                                         debug=debug)

    @classmethod
    async def search_datasets(cls,
                              full_auth=dmda.DomoFullAuth,
                              body: dict = None,
                              session: aiohttp.ClientSession = None,
                              maximum: int = None,
                              debug=False) -> list[DomoDataset]:

        if not body:
            body = DomoDatacenter.generate_search_datacenter_body(
                
                entities_list=['DATASET'],
                filters=[],
                count=1000,
                offset=0,
                combineResults=False
            )

        if debug:
            print(body)

        json_list = await cls.search_datacenter(full_auth=full_auth,
                                                maximum=maximum,
                                                body=body,
                                                session=session,
                                                debug=False)

        if json_list:
            return await asyncio.gather(
                *[DomoDataset.get_from_id(id=json_obj.get('databaseId'), full_auth=full_auth, debug=debug)
                  for json_obj in json_list]
            )

    @classmethod
    async def get_lineage_upstream(cls,
                                   full_auth: dmda.DomoFullAuth,
                                   entity_id,
                                   entity_type,
                                   session: aiohttp.ClientSession = None,
                                   debug: bool = False,
                                   debug_prn: bool = False):

        import Library.DomoClasses.DomoDataflow as dmdf
        import Library.DomoClasses.DomoDataset as dmds

        try:
            if not session:
                session = aiohttp.ClientSession()
                is_close_session = True

            res = await datacenter_routes.get_lineage_upstream(full_auth=full_auth,
                                                               entity_type=entity_type,
                                                               entity_id=entity_id,
                                                               session=session, debug=debug)

            if res.status == 200:
                obj = res.response

                domo_obj = []
                for key, item in obj.items():
                    if item.get('type') == 'DATA_SOURCE':
                        domo_obj.append(await dmds.DomoDataset.get_from_id(full_auth=full_auth, id=item.get('id')))

                    if item.get('type') == 'DATAFLOW':
                        # print(item.get('id'))
                        domo_obj.append(await dmdf.DomoDataflow.get_from_id(full_auth=full_auth, id=item.get('id')))
                        pass

                return domo_obj
            else:
                return None

        finally:
            if is_close_session:
                await session.close()

    @classmethod
    async def get_accounts(cls, full_auth: dmda.DomoFullAuth,
                           search_str: str = None,
                           is_exact_match: bool = True,
                           maximum: int = None,
                           session: aiohttp.ClientSession = None, debug: bool = False):

        import Library.DomoClasses.DomoAccount as dma

        json_ls = None

        if search_str:
            body = datacenter_routes.generate_search_datacenter_account_body(
                search_str=search_str, is_exact_match=is_exact_match)

            search_ls = await cls.search_datacenter(full_auth=full_auth,
                                                    maximum=maximum,
                                                    body=body,
                                                    session=session,
                                                    debug=debug)

            if not search_ls or len(search_ls) == 0:
                return None

            json_ls = [{'id': obj.get('databaseId')} for obj in search_ls]

        else:
            res = await account_routes.get_accounts(full_auth=full_auth, session=session, debug=debug)
            
            if debug:
                print(res)

            if res.status != 200:
                return None

            json_ls = res.response

        domo_account_ls = await asyncio.gather(*[dma.DomoAccount.get_from_id(account_id=obj.get('id'), full_auth=full_auth) for obj in json_ls])

        if is_exact_match and search_str:
            return next((domo_account for domo_account in domo_account_ls if domo_account.name == search_str), None)

        return domo_account_ls



import datetime as dt
from dataclasses import dataclass
from enum import Enum

from ..utils import convert as cd
from ..utils.DictDot import DictDot


class DomoCertificationState(Enum):
    CERTIFIED = 'certified'


@dataclass
class DomoCertification:
    certification_state: DomoCertificationState
    last_updated: dt.datetime
    certification_type: str
    certification_name: str

    @classmethod
    def _from_json(cls, dd):
        return cls(certification_state=DomoCertificationState[dd.state].value or dd.state,
                   last_updated=cd.convert_epoch_millisecond_to_datetime(
                       dd.lastUpdated),
                   certification_type=dd.processType,
                   certification_name=dd.processName
                   )



import aiohttp
import pandas as pd
import asyncio

import Library.utils.LoggerClass as lc

async def upload_data(instance_auth,
                      consol_auth,
                      data_fn,
                      consol_ds,
                      is_index: bool = False,
                      debug: bool = False,
                      debug_prn: bool = False,
                      logger: lc.MyLogger = None):

    try:
        # await asyncio.sleep(randrange(5))
        if logger : 
            logger.log_info (f" Upload_data function - starting {instance_auth.domo_instance} - {data_fn.__name__}")
        if debug_prn:
            print(
                f"starting {instance_auth.domo_instance} - {data_fn.__name__}")

        instance_session = aiohttp.ClientSession()

        upload_df = await data_fn(instance_auth, instance_session)

        if upload_df is None or len(upload_df.index) == 0:
            return None

        await instance_session.close()

        return await consol_ds.upload_csv(upload_df=upload_df,
                                          full_auth=consol_auth,
                                          upload_method='APPEND',
                                          partition_key=instance_auth.domo_instance,
                                          is_index=is_index)

    except Exception as e:
        print(f"upload_data : unexpected error: {e}")
        if logger : 
            logger.log_error(f"upload_data : unexpected error: {e}")
        return None

    finally:
        await instance_session.close()


async def upload_data_with_date(instance_auth,
                                consol_auth,
                                data_fn,
                                consol_ds,
                                partition_date_col,
                                partition_delimiter,
                                start_date,
                                end_date,
                                debug: bool = False,
                                debug_prn: bool = False):

    instance_session = aiohttp.ClientSession()

    print(
        f"'🎬 upload_with_data: starting retrieval {start_date}, {end_date}, {instance_auth.domo_instance}")

    upload_df = await data_fn(instance_auth=instance_auth,
                              session=instance_session,
                              start_date=start_date,
                              end_date=end_date,
                              debug=debug)

    await instance_session.close()

    if not isinstance(upload_df, pd.DataFrame):
        print(f"🛑 error no data returned {instance_auth.domo_instance}")
        print(upload_df)
        return None

    if debug_prn:
        print(
            f'🧻 upload_with_data: starting upload {len(upload_df)} rows for {instance_auth.domo_instance}')

    task = []

    for index, partition_set in upload_df.drop_duplicates(subset=[partition_date_col]).iterrows():
        partition_date = partition_set[partition_date_col]

        partition_key = f"{instance_auth.domo_instance}{partition_delimiter}{str(partition_date)}"

        task.append(consol_ds.upload_csv(upload_df=upload_df[(upload_df[partition_date_col] == partition_date)],
                                         upload_method='REPLACE',
                                         debug=debug,
                                         partition_key=partition_key,
                                         is_index=False))

    res = await asyncio.gather(*task)

    if debug_prn:
        print(
            f'🎉 upload_with_data : finished uploading {len(upload_df)} rows for {instance_auth.domo_instance}')
    return res



import json
from enum import Enum

import Transport as tr


class Snowflake_DatabaseObjects(Enum):
    TABLES = 'tables'


def con_snowflake(warehouse_name,
                  database_name,
                  schema_name,
                  table_name,
                  database_objects: Snowflake_DatabaseObjects = Snowflake_DatabaseObjects.TABLES,

                  qry_whereClause=None,
                  qry_limitClause: str = None,

                  account_id=None,
                  ds_name=None,
                  ds_description=None,
                  query=None,
                  schedule: tr.Schedule = None
                  ):
    cnfg = tr.Config.SNOWFLAKE_UNLOADv2.value

    if not query:
        query = f"SELECT * \nFROM {table_name} \n{qry_whereClause} "
        if qry_limitClause:
            query += f"\n{qry_limitClause}"

    configuration = [
        tr.ConfigurationParam(
            name='importDataMethod',
            value="noPartiionAndUpsert"),
        tr.ConfigurationParam(
            name='warehouseName',
            value=warehouse_name),
        tr.ConfigurationParam(
            name='databaseName',
            value=database_name),
        tr.ConfigurationParam(
            name='schemaName',
            value=schema_name),
        tr.ConfigurationParam(
            name='databaseObjects',
            value=database_objects.value),
        tr.ConfigurationParam(
            name='tableName',
            value=table_name),
        tr.ConfigurationParam(
            name='queryType',
            value='customQuery'),
        tr.ConfigurationParam(
            name='query',
            value=query),

    ]

    config = tr.Configuration(transport=tr.Transport(type=cnfg.get('type'),
                                                     description=cnfg.get(
                                                         'description'),
                                                     version=cnfg.get('version')),
                              account=tr.Account(id=account_id),
                              configuration=configuration,
                              updateMethod=tr.UpdateMethod.APPEND.value,
                              dataProvider=tr.DataProvider(
                                  key=cnfg.get('dataProvider_key')),
                              dataSource=tr.Datasource(
                                  name=ds_name, description=ds_description),
                              advancedScheduleJson=schedule
                              )

    return config


cnfg_pgsql = con_snowflake(warehouse_name="WH_BIS_PARTNER_ANALYTICS_HEAVY_NONPROD",
                           database_objects=Snowflake_DatabaseObjects.TABLES,
                           schema_name="PUBLIC",
                           database_name="BIS_PRD_01_REF",
                           table_name="HASH_ACCOUNT_ID_MAPPING",
                           account_id=3,
                           ds_name='hello world',
                           ds_description='config works?',

                           query="SELECT \nha.\"ACCT_ID\", \nha.\"HASH_ACCT_ID\" ,\nem.\"HASHED_EMAIL_ADDRESS\"\n\nFROM \"BIS_PRD_01_REF\".\"PUBLIC\".\"HASH_ACCOUNT_ID_MAPPING\" ha\n\nINNER JOIN PS_PRD_01_USERODS.PUBLIC.CCRM_ACCT_HASHED_EMAIL_MAP em\n\nON\n\nha.\"ACCT_ID\" = em.\"ACCT_ID\""
                           )

# pprint(cnfg_pgsql.to_dict())


print(json.dumps(cnfg_pgsql.to_dict()))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/utils/convert.ipynb.

# %% auto 0
__all__ = ['convert_epoch_millisecond_to_datetime', 'convert_datetime_to_epoch_millisecond', 'convert_snake_to_pascal',
           'ConcatDataframe_InvalidElement', 'concat_list_dataframe']

# %% ../../nbs/utils/convert.ipynb 2
import datetime as dt
import pandas as pd

# %% ../../nbs/utils/convert.ipynb 5
def convert_epoch_millisecond_to_datetime(epoch: int):
    '''convert Epoch time with miliseconds to Date time'''
    return dt.datetime.fromtimestamp(epoch / 1000.0)

# %% ../../nbs/utils/convert.ipynb 6
def convert_datetime_to_epoch_millisecond(datetime: dt.datetime):
    '''convert DateTime to Epoch time with Miliseconds'''
    return int(datetime.timestamp() * 1000)

# %% ../../nbs/utils/convert.ipynb 10
def convert_snake_to_pascal(str):
    """converts 'snake_case_str' to 'snakeCaseStr'"""
    
    clean_str = str.replace("_", " ").title().replace(" ", "")
    return clean_str[0].lower()+clean_str[1:]

# %% ../../nbs/utils/convert.ipynb 14
class ConcatDataframe_InvalidElement(Exception):
    def __init__(self, elem):
        message = f"{type(elem)} passed into dataframe"
        super().__init__(message)

def concat_list_dataframe(df_ls : list[pd.DataFrame]) -> pd.DataFrame:
    """take a list of dataframes and collapse into one dataframe"""

    df = None
    for elem in df_ls:
        if not isinstance(elem, pd.DataFrame):
            raise ConcatDataframe_InvalidElement(elem)
        
        if len(elem.index) == 0:
            pass
        
        if df is None:
            df = elem

        else:
            df = pd.concat([df, elem], join = 'inner').reset_index(drop = True)
    return df
